import xlrd
import pandas as pd 
import time
from selection import selection

def createModelData(path_in, year):
    #Creates all input data for reduced model
    wb = xlrd.open_workbook(path_in)
    ws1 = wb.sheet_by_name('Input_Tech')
    ws2 = wb.sheet_by_name('Input_Parameters')
    
    di = {}
    #Sets:
    di['T'] = {None: list(range(0, int(ws2.cell(16, 2).value) + 1))}
    di['lastT'] = {None: (ws2.cell(16, 2).value)}
    di['ST'] = {None:[1, 2]}

    for row in (15, 16, 17, 18, 19, 20, 21, 22, 26, 27, 28, 29, 30, 31, 32, \
                33, 34, 35, 36, 37, 39, 40, 41, 42, 45, 46, 47, 52, 53, \
                54, 57, 59, 60, 61):
        if row in [15, 16, 21, 22, 26, 30, 33, 39]:
            di[ws2.cell(row,1).value] = {None: int(ws2.cell(row,2).value)}

        else:                
            di[ws2.cell(row,1).value] = {None: ws2.cell(row,2).value}
    
    #important parameters:   
    I = []
    for row in range(11):
        I.append(ws1.cell(4+row,1).value)
    di['I'] = {None:I}

    for col in (2, 3, 4, 5, 6, 7, 8, 9):
        data = {}
        for row in range(11):
            if col in [2,6]:
                data[I[row]] = int(ws1.cell(4+row,col).value)
            else:
                data[I[row]] = ws1.cell(4+row,col).value
            di[ws1.cell(2, col).value] = data
    
    if ws2.cell(16, 2).value == 8735:
        df = pd.read_excel('Scenario_data.xlsx', str(year), usecols =  [3,4,5,6,7,8,18,19,20,21]) #Now with separated COPs for DHW and SH
        df = df.drop([0])
        df = df.drop(range(8737,8761))
        new_df = pd.DataFrame(data=df.values, columns = ['D_el','D_sh','D_dhw','P_spot','Temp','Y_pv','COP_ashp_sh', 'COP_ashp_wh','COP_gshp_sh', 'COP_gshp_wh'], index = range(8736))
        data = new_df.to_dict()
        
        di['D_el'] = data['D_el']
        di['D_sh'] = data['D_sh']
        di['D_dhw'] = data['D_dhw']
        di['P_spot'] = data['P_spot']
        di['Temp'] = data['Temp']
        di['Y_pv'] = data['Y_pv']
        di['COP_ashp_sh'] = data['COP_ashp_sh']
        di['COP_ashp_dhw'] = data['COP_ashp_wh']
        di['COP_gshp_sh'] = data['COP_gshp_sh']
        di['COP_gshp_dhw'] = data['COP_gshp_wh']
        
        return di
    
    elif ws2.cell(16, 2).value == 671:
        #select 4 weeks for each season surrounding the highest space heating load, week size is 168 hours
        df = pd.read_excel('Scenario_data.xlsx', str(year), usecols =  [3,4,5,6,7,8,18,19,20,21]) #Now with separated COPs for DHW and SH
        df = df.drop([0])
        df = df.drop(range(8737,8761))
        select_dict = selection(str(year), 'Scenario_data.xlsx')
        df = df.drop(range(1,select_dict['winter_start']))
        df = df.drop(range(select_dict['winter_start'] + 168 ,select_dict['spring_start']))
        df = df.drop(range(select_dict['spring_start'] + 168 ,select_dict['summer_start']))
        df = df.drop(range(select_dict['summer_start'] + 168 ,select_dict['fall_start']))
        df = df.drop(range(select_dict['fall_start'] + 168 , 8737))
        
        new_df = pd.DataFrame(data=df.values, columns = ['D_el','D_sh','D_dhw','P_spot','Temp','Y_pv','COP_ashp_sh', 'COP_ashp_wh','COP_gshp_sh', 'COP_gshp_wh'], index = range(672))
        data = new_df.to_dict()
        
        di['D_el'] = data['D_el']
        di['D_sh'] = data['D_sh']
        di['D_dhw'] = data['D_dhw']
        di['P_spot'] = data['P_spot']
        di['Temp'] = data['Temp']
        di['Y_pv'] = data['Y_pv']
        di['COP_ashp_sh'] = data['COP_ashp_sh']
        di['COP_ashp_dhw'] = data['COP_ashp_wh']
        di['COP_gshp_sh'] = data['COP_gshp_sh']
        di['COP_gshp_dhw'] = data['COP_gshp_wh']
        
        return di
   
def saveDeterministicResults(m, result_file):

    from openpyxl import load_workbook
    
    wb = load_workbook(result_file)
    ser = wb['Series']
    val = wb['Values']
    if m.lastT == 23:
        f = 8736/24
    elif m.lastT == 287:
        f = 8736/288
    elif m.lastT == 8735:
        f = 1
    elif m.lastT==671:
        f = 13
 
    for t in m.T:             
        for i, elem in enumerate(['D_el', 'y_imp', 'y_exp', 'y_pty', 'y_ch', 'y_dch', \
                                  'z_ba', 'y_pv', 'y_ashp_sh', 'y_ashp_dhw', 'y_gshp_sh',\
                                  'y_gshp_dhw', 'y_eb_sh','y_eb_dhw', 'D_sh', 'D_dhw', \
                                  'q_hs', 'q_ashp_sh','q_ashp_dhw', 'q_gshp_sh','q_gshp_dhw',\
                                  'q_eb_sh', 'q_eb_dhw', 'z_hs', 'q_ss', 'z_ss','z_ds', 'z_hwt', 'q_hwt', 'q_bb_sh', 'q_bb_dhw', 'Flow']):
            ser.cell(row = 1, column = i + 1, value = elem)
        
        ser.cell(row= t+2, column=1, value=m.D_el[t])
        ser.cell(row= t+2, column=2, value=m.y_imp[t].value)
        ser.cell(row= t+2, column=3, value=m.y_exp[t].value)
        ser.cell(row= t+2, column=4, value=m.y_pty[t].value)
        
        ser.cell(row= t+2, column=5, value=m.y_ch[t].value)
        ser.cell(row= t+2, column=6, value=m.y_dch[t].value)
        ser.cell(row= t+2, column=7, value=m.z_ba[t].value)
        
        
        ser.cell(row= t+2, column=8, value=m.y_pv[t].value)
        ser.cell(row= t+2, column=9, value=m.y_ashp_sh[t].value)
        ser.cell(row= t+2, column=10, value=m.y_ashp_dhw[t].value)
        ser.cell(row= t+2, column=11, value=m.y_gshp_sh[t].value)
        ser.cell(row= t+2, column=12, value=m.y_gshp_dhw[t].value)
        ser.cell(row= t+2, column=13, value=m.y_eb_sh[t].value)
        ser.cell(row= t+2, column=14, value=m.y_eb_dhw[t].value)
        ser.cell(row= t+2, column=15, value=m.D_sh[t])
        ser.cell(row= t+2, column=16, value=m.D_dhw[t])
        ser.cell(row= t+2, column=17, value=m.q_hs[t].value) #HS output
        ser.cell(row= t+2, column=18, value=m.q_ashp_sh[t].value) 
        ser.cell(row= t+2, column=19, value=m.q_ashp_dhw[t].value) 
        ser.cell(row= t+2, column=20, value=m.q_gshp_sh[t].value)
        ser.cell(row= t+2, column=21, value=m.q_gshp_dhw[t].value) 
        ser.cell(row= t+2, column=22, value=m.q_eb_sh[t].value) 
        ser.cell(row= t+2, column=23, value=m.q_eb_dhw[t].value) 
        ser.cell(row= t+2, column=24, value=m.z_hs[t].value)
        ser.cell(row= t+2, column=25, value=m.q_ss[t].value)
        ser.cell(row= t+2, column=26, value=m.z_ss[t].value) 
        ser.cell(row= t+2, column=27, value=m.z_ds[t].value)
        ser.cell(row= t+2, column=28, value=m.z_hwt[t].value) 
        ser.cell(row= t+2, column=29, value=m.q_hwt[t].value)
        ser.cell(row= t+2, column=30, value=m.q_bb_sh[t].value) 
        ser.cell(row= t+2, column=31, value=m.q_bb_dhw[t].value) 
        ser.cell(row= t+2, column=32, value=m.Flow[t].value) 
    
    #Invested capacities
    val['A1'] = 'Time' 
    val['B1'] = time.ctime() #Time and date
    #val['C4'] = time.time() - start_time #Code run time
    val['A2'] = 'Scenario name'
    val['B2'] = 'NOR'
    val['A3'] = 'gamma'
    val['B3'] = m.gamma.value
    val['A4'] = 'A_co2'
    val['B4'] = m.A_co2.value
    val['A5'] = 'A_pe'
    val['B5'] = m.A_pe.value
    
    val['B6'] = m.x['PV'].value
    val['B7'] = m.x['BA'].value
    val['B8'] = m.x['DH'].value
    val['B9'] = m.x['EB'].value
    val['B10'] = m.x['BB'].value
    val['B11'] = m.x['GSHP'].value
    val['B12'] = m.x['ASHP'].value
    val['B13'] = m.x['HS'].value
    val['B14'] = m.SS_cap.value
    val['B15'] = m.DS_cap.value
    val['B16'] = m.x['HWT'].value
    #val['B15'] = m.x['SS'].value
    
    val['A6'] = 'pv'
    val['A7'] = 'ba'
    val['A8'] = 'dh'
    val['A9'] = 'eb'
    val['A10'] = 'bb'
    val['A11'] = 'gshp'
    val['A12'] = 'ashp'
    val['A13'] = 'hs'
    val['A14'] = 'ss'
    val['A15'] = 'ds'
    val['A16'] = 'hwt'
    #val['A15'] = 'ss'  

    val['C5'] = 'Tech inv cost'
    val['C6'] = (m.C_spe['PV']*m.x['PV'].value+ m.C_fxd['PV']*m.a_i['PV'].value)
    val['C7'] = (m.C_spe['BA']*m.x['BA'].value+ m.C_fxd['BA']*m.a_i['BA'].value)
    val['C8'] = (m.C_spe['DH']*m.x['DH'].value+ m.C_fxd['DH']*m.a_i['DH'].value)
    val['C9'] = (m.C_spe['EB']*m.x['EB'].value+ m.C_fxd['EB']*m.a_i['EB'].value)
    val['C10'] = (m.C_spe['BB']*m.x['BB'].value+ m.C_fxd['BB']*m.a_i['BB'].value)
    val['C11'] = (m.C_spe['GSHP']*m.x['GSHP'].value+ m.C_fxd['GSHP']*m.a_i['GSHP'].value)
    val['C12'] = (m.C_spe['ASHP']*m.x['ASHP'].value+ m.C_fxd['ASHP']*m.a_i['ASHP'].value)
    val['C13'] = (m.C_spe['HS']*m.x['HS'].value+ m.C_fxd['HS']*m.a_i['HS'].value)
    val['C14'] = 0
    val['C15'] = 0
    val['C16'] = (m.C_spe['HWT']*m.x['HWT'].value+ m.C_fxd['HWT']*m.a_i['HWT'].value)
        
    val['D1'] = m.objective_TotalCost()
    val['D2'] = m.cost_Investments[1]()
    val['D3'] = m.cost_Operations[2]()
    
    val['C1'] = 'tot_cost'
    val['C2'] = 'inv_cost'
    val['C3'] = 'op_cost'
    
    y_imp_tot = 0
    y_exp_tot = 0
    bf_tot = 0
    for t in m.T:
        y_imp_tot += m.y_imp[t].value
        y_exp_tot += m.y_exp[t].value
        bf_tot += m.bf_sh[t].value + m.bf_dhw[t].value
        
    G_tot = m.G_bf*bf_tot + m.G_el*y_imp_tot
    
    val['F1'] = 'y_imp_tot'
    val['F2'] = 'y_exp_tot'
    val['F3'] = 'bf_tot'
    val['F4'] = 'G_tot'
    
    val['G1'] = y_imp_tot
    val['G2'] = y_exp_tot
    val['G3'] = bf_tot
    val['G4'] = G_tot
    
    val['F8'] = 'BITES-params'
    val['F9'] = 'K_flow'
    val['F10'] = 'K_shallow'
    val['F11'] = 'K_deep'
    
    val['G9'] = m.K_flow.value
    val['G10'] = m.K_shallow.value
    val['G11'] = m.K_deep.value
    
    y_imp_max = 0
    for t in m.T:
        if m.y_imp[t].value > y_imp_max:
            y_imp_max = m.y_imp[t].value
    
    val['F13'] = 'peak load el'
    val['G13'] = y_imp_max

    wb.save(result_file)    
    print('Deterministic result file successfully created')